--[[ 
	Amanuensis (n.) - a person employed to write what another dictates or to copy what has been written by another; secretary.
	
	Goal:
	*  Intercept and respond to whispers during important events (ex. boss encounters, arenas, etc.)
	-    Hide whispers from user until encounter is complete.
	*  Provide an extended 'Ignore' list.
]]


Amanuensis = LibStub('AceAddon-3.0'):NewAddon('Amanuensis', 'AceEvent-3.0', 'AceHook-3.0', 'AceConsole-3.0')


function Amanuensis:OnEnable()
	local _G = _G
	local strformat = _G.string.format
	
	_G.AmanuensisDB = _G.AmanuensisDB or {
		msgHolding = {
			hold = false,			-- Currently holding messages or not.
			reminderInterval = 5,		-- The interval between reminders.
			senderNotification = true	-- Notify the sender that their message will be delivered at a later time.
		}
	}
	
	-- Stores a list of all the chatframes that the user wants whispers to appear in.
	self.RegisteredChatFrames = {}
	
	-- Unregister whispers from Blizzard's chat frames.  When a whisper is received and
	-- the user isn't holding messages, we can simulate the event by calling the
	-- event handler exclusively.
	self:OnChatPreferencesChanged()
	
	-- These functions are called whenever the player changes which chatframes messages
	-- are displayed in.
	self:Hook('ChatFrame_AddMessageGroup', 'OnChatPreferencesChanged', true)
	self:Hook('ChatFrame_RemoveMessageGroup', 'OnChatPreferencesChanged', true)
	
	self:RegisterEvent('CHAT_MSG_WHISPER')
	
	self:RegisterChatCommand('amanuensis', 'OnSlashCommand')
	self:RegisterChatCommand('ams', 'OnSlashCommand')
end


function Amanuensis:OnSlashCommand(msg)
	if msg == 'hold silent' then
		self.msgHolding.senderNotification = not self.msgHolding.senderNotification
		self:Print(string.format('Sender Notification is now |cff%s|r.',
						 self.msgHolding.senderNotification and '00ff00ON' or
						 'ff0000OFF'))
	elseif msg == 'hold' then
		self.msgHolding.hold = not self.msgHolding.hold
		
	else
		
	end
end

-- arg1 = msg, arg2 = author, arg6 = status ("DND", "GM")
function Amanuensis:CHAT_MSG_WHISPER(...)
	if select(6, ...) == 'GM' then
		-- bypass this function completely.
	end
end


function Amanuensis:HoldMessages(silent)
	-- silent: If 'false' (default), let the sender know that their message will be relayed to the user at a later time.
end


function Amanuensis:OnChatPreferencesChanged()
	local strformat = string.format
	for i = 1, 7 do
		if _G[strformat("ChatFrame%d", i)]:IsEventRegistered("CHAT_MSG_WHISPER") then
			self.RegisteredChatFrames[i] = true
		else
			self.RegisteredChatFrames[i] = nil
		end
	end
end
